/*
 * JBoss, the OpenSource J2EE webOS
 * 
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.fedorahosted.tennera.antgettext;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.util.Collection;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.DirectoryScanner;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.taskdefs.MatchingTask;
import org.fedorahosted.openprops.Properties;
import org.fedorahosted.tennera.jgettext.Catalog;
import org.fedorahosted.tennera.jgettext.Message;
import org.fedorahosted.tennera.jgettext.Occurence;
import org.fedorahosted.tennera.jgettext.Catalog.MessageProcessor;
import org.fedorahosted.tennera.jgettext.catalog.parse.ExtendedCatalogParser;

/**
 * 
 * 
 * @author <a href="sflaniga@redhat.com">Sean Flanigan</a>
 * @version $Revision: $
 */
public class Po2PropTask extends MatchingTask
{
   private static final boolean INCLUDE_PROCESSING_COMMENT = false;
   // TODO disable this once multiline comments are fixed in prop2po
   private static final boolean INCLUDE_MESSAGE_COMMENTS = true;
   private File srcDir;
   private File dstDir;
   // In future, we might use the original English properties as a template 
   // as Translate Toolkit's po2prop does.   po2prop may use templates to 
   // (a) preserve ordering, and (b) find the ResourceBundle key (in case 
   // the #: location comments have been removed).  
//   private File tmpDir;
   private String locale = null;

   public void setSrcDir(File srcDir)
   {
      this.srcDir = srcDir;
   }

//   public void setTmpDir(File tmpDir)
//   {
//      this.tmpDir = tmpDir;
//   }
   
   public void setDstDir(File dstDir)
   {
      this.dstDir = dstDir;
   }
   
   public void setLocale(String locale)
   {
       this.locale = locale;
   }
   
   @Override
   public void execute() throws BuildException
   {
      DirUtil.checkDir(srcDir, "srcDir", false);
      DirUtil.checkDir(dstDir, "dstDir", true);
//      checkDir(tmpDir, "tmpDir");

      try
      {
         DirectoryScanner ds = super.getDirectoryScanner(srcDir);
         // use default includes if unset:
         if(!getImplicitFileSet().hasPatterns())
             ds.setIncludes(new String[] {"**/*.po"});
         ds.scan();
         String[] files = ds.getIncludedFiles();

         for (int i = 0; i < files.length; i++)
         {
            String poFilename = files[i];
            File poFile = new File(srcDir, poFilename);
            String localeSuffix;
            if (locale == null || locale.length() == 0)
		localeSuffix = "";
	    else
		localeSuffix = "_" + locale;
            String basename = poFilename.substring(0, poFilename.length()-".po".length());
	    String propFilename = basename + localeSuffix + ".properties";
            File propFile = new File(dstDir, propFilename);
            final Properties props = new Properties();
            log("Generating "+propFile+" from "+poFile, Project.MSG_VERBOSE);

            propFile.getParentFile().mkdirs();
            BufferedOutputStream out = new BufferedOutputStream(new FileOutputStream(propFile));
            try
            {
//               POUnmarshaller unmarshaller = POFactory.createUnmarshaller();
//               POFile file = unmarshaller.unmarshall(poFile);
//               for (POEntry entry : file.getEntries())
		ExtendedCatalogParser parser = new ExtendedCatalogParser( poFile );
		parser.catalog();
		Catalog catalog = parser.getCatalog();
		MessageProcessor processor = new MessageProcessor() 
		{
		    public void processMessage(Message entry) {
			String ctxt = entry.getMsgctxt();
			// NB entries which don't have (a) ctxt or (b) reference will be ignored
			Collection<String> poComments = entry.getExtractedComments();
			StringBuilder sb = new StringBuilder();
			for (String comm : poComments) 
			{
			    sb.append(comm).append("\n");
			}
			String poComment = sb.toString();
			//System.err.println("Processing "+entry.toString());
			if (ctxt == null)
			{
			    // TODO gettext tools and the POUnmarshaller do not preserve whitespace in references 
			    // so we should use the original en properties as a template
			    if(entry.getMsgid().length() != 0)
			    {
				for (Occurence occ : entry.getOccurences())
				{
				    String ref = occ.toString();
				    props.setProperty(ref, entry.getMsgstr());
				    if (poComment.length() != 0 && INCLUDE_MESSAGE_COMMENTS)
					props.setComment(ref, poComment);
				}
			    }
			}
			else
			{
			    props.setProperty(ctxt, entry.getMsgstr());
			    if (poComment.length() != 0 && INCLUDE_MESSAGE_COMMENTS)
				props.setComment(ctxt, poComment);
			}
		    }
		};
		catalog.processMessages( processor);

		String comment = null;
		if (INCLUDE_PROCESSING_COMMENT) {
		    comment = propFilename+" generated by "+Po2PropTask.class.getName()+" from "+poFilename;
		}
		props.store(out, comment);
//               props.store(System.out, comment);
            }
            finally
            {
               out.close();
            }
         }
      }
      catch (Exception e)
      {
         throw new BuildException(e);
      }
   }

}
